@@ -20,15 +20,13 @@ package org.xnio;
 
 import java.io.Closeable;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.RandomAccessFile;
 import java.lang.management.ManagementFactory;
 import java.net.InetSocketAddress;
 import java.nio.channels.FileChannel;
 import java.nio.file.NoSuchFileException;
+import java.nio.file.Path;
 import java.nio.file.StandardOpenOption;
 import java.security.AccessController;
 import java.security.GeneralSecurityException;
@@ -41,6 +39,11 @@ import java.util.concurrent.atomic.AtomicBoolean;
 
 import javax.management.MBeanServer;
 import javax.management.ObjectName;
+
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleContext;
+import org.osgi.framework.FrameworkUtil;
+import org.osgi.framework.ServiceReference;
 import org.xnio.management.XnioProviderMXBean;
 import org.xnio.management.XnioServerMXBean;
 import org.xnio.management.XnioWorkerMXBean;
@@ -83,19 +86,12 @@ public abstract class Xnio {
     }
 
     /**
-     * A flag indicating the presence of NIO.2 (JDK 7).
+     * A flag indicating the presence of NIO.2 (JDK 7).  Always {@code true} as of XNIO version 3.5.0, which requires
+     * Java 8.
      */
-    public static final boolean NIO2;
+    public static final boolean NIO2 = true;
 
     static {
-        boolean nio2 = false;
-        try {
-            // try to find an NIO.2 interface on the system class path
-            Class.forName("java.nio.channels.MulticastChannel", false, null);
-            nio2 = true;
-        } catch (Throwable t) {
-        }
-        NIO2 = nio2;
         msg.greeting(Version.VERSION);
         final EnumMap<FileAccess, OptionMap> map = new EnumMap<FileAccess, OptionMap>(FileAccess.class);
         for (FileAccess access : FileAccess.values()) {
@@ -252,9 +248,36 @@ public abstract class Xnio {
                 msg.debugf(t, "Skipping non-loadable provider");
             }
         }
+        try {
+            Xnio xnio = OsgiSupport.doGetOsgiService();
+            if (xnio != null) {
+                return xnio;
+            }
+        } catch (NoClassDefFoundError t) {
+            // Ignore
+        } catch (Throwable t) {
+            msg.debugf(t, "Not using OSGi service");
+        }
         throw msg.noProviderFound();
     }
 
+    static class OsgiSupport {
+
+        static Xnio doGetOsgiService() {
+            Bundle bundle = FrameworkUtil.getBundle(Xnio.class);
+            BundleContext context = bundle.getBundleContext();
+            if (context == null) {
+                throw new IllegalStateException("Bundle not started");
+            }
+            ServiceReference<Xnio> sr = context.getServiceReference(Xnio.class);
+            if (sr == null) {
+                return null;
+            }
+            return context.getService(sr);
+        }
+
+    }
+
     //==================================================
     //
     // SSL methods
@@ -291,70 +314,6 @@ public abstract class Xnio {
     //
     //==================================================
 
-    private interface Opener {
-        FileChannel openFile(File file, OptionMap options) throws IOException;
-    }
-
-    private static final Opener OPENER = NIO2 ? new Nio2Opener() : new Nio1Opener();
-
-    private static final class Nio1Opener implements Opener {
-        public FileChannel openFile(final File file, final OptionMap options) throws IOException {
-            final FileAccess fileAccess = options.get(Options.FILE_ACCESS, FileAccess.READ_WRITE);
-            final boolean append = options.get(Options.FILE_APPEND, false);
-            final boolean create = options.get(Options.FILE_CREATE, fileAccess != FileAccess.READ_ONLY);
-            if (fileAccess == FileAccess.READ_ONLY) {
-                if (append) {
-                    throw msg.readAppendNotSupported();
-                }
-                if (create) {
-                    throw msg.openModeRequires7();
-                }
-                return new XnioFileChannel(new FileInputStream(file).getChannel());
-            } else if (fileAccess == FileAccess.READ_WRITE) {
-                if (append) {
-                    throw msg.openModeRequires7();
-                }
-                if (! create) {
-                    throw msg.openModeRequires7();
-                }
-                return new XnioFileChannel(new RandomAccessFile(file, "rw").getChannel());
-            } else if (fileAccess == FileAccess.WRITE_ONLY) {
-                if (! create) {
-                    throw msg.openModeRequires7();
-                }
-                return new XnioFileChannel(new FileOutputStream(file, append).getChannel());
-            } else {
-                throw new IllegalStateException();
-            }
-        }
-    }
-
-    private static final class Nio2Opener implements Opener {
-        public FileChannel openFile(final File file, final OptionMap options) throws IOException {
-            try {
-                final FileAccess fileAccess = options.get(Options.FILE_ACCESS, FileAccess.READ_WRITE);
-                final boolean append = options.get(Options.FILE_APPEND, false);
-                final boolean create = options.get(Options.FILE_CREATE, fileAccess != FileAccess.READ_ONLY);
-                final EnumSet<StandardOpenOption> openOptions = EnumSet.noneOf(StandardOpenOption.class);
-                if (create) {
-                    openOptions.add(StandardOpenOption.CREATE);
-                }
-                if (fileAccess.isRead()) {
-                    openOptions.add(StandardOpenOption.READ);
-                }
-                if (fileAccess.isWrite()) {
-                    openOptions.add(StandardOpenOption.WRITE);
-                }
-                if (append) {
-                    openOptions.add(StandardOpenOption.APPEND);
-                }
-                return new XnioFileChannel(FileChannel.open(file.toPath(), openOptions.toArray(new StandardOpenOption[openOptions.size()])));
-            } catch (NoSuchFileException e) {
-                throw new FileNotFoundException(e.getMessage());
-            }
-        }
-    }
-
     /**
      * Open a file on the filesystem.
      *
@@ -370,7 +329,28 @@ public abstract class Xnio {
         if (options == null) {
             throw msg.nullParameter("options");
         }
-        return OPENER.openFile(file, options);
+        try {
+            final FileAccess fileAccess = options.get(Options.FILE_ACCESS, FileAccess.READ_WRITE);
+            final boolean append = options.get(Options.FILE_APPEND, false);
+            final boolean create = options.get(Options.FILE_CREATE, fileAccess != FileAccess.READ_ONLY);
+            final EnumSet<StandardOpenOption> openOptions = EnumSet.noneOf(StandardOpenOption.class);
+            if (create) {
+                openOptions.add(StandardOpenOption.CREATE);
+            }
+            if (fileAccess.isRead()) {
+                openOptions.add(StandardOpenOption.READ);
+            }
+            if (fileAccess.isWrite()) {
+                openOptions.add(StandardOpenOption.WRITE);
+            }
+            if (append) {
+                openOptions.add(StandardOpenOption.APPEND);
+            }
+            final Path path = file.toPath();
+            return new XnioFileChannel(path.getFileSystem().provider().newFileChannel(path, openOptions));
+        } catch (NoSuchFileException e) {
+            throw new FileNotFoundException(e.getMessage());
+        }
     }
 
     /**
@@ -441,6 +421,23 @@ public abstract class Xnio {
     //
     //==================================================
 
+    /**
+     * Create a new worker builder.
+     *
+     * @return the worker builder (not {@code null})
+     */
+    public XnioWorker.Builder createWorkerBuilder() {
+        return new XnioWorker.Builder(this);
+    }
+
+    /**
+     * Construct an XNIO worker from a builder.
+     *
+     * @param builder the builder (must not be {@code null})
+     * @return the constructed worker
+     */
+    protected abstract XnioWorker build(XnioWorker.Builder builder);
+
     /**
      * Construct a new XNIO worker.
      *
@@ -476,7 +473,13 @@ public abstract class Xnio {
      * @throws IOException if the worker failed to be opened
      * @throws IllegalArgumentException if an option value is invalid for this worker
      */
-    public abstract XnioWorker createWorker(ThreadGroup threadGroup, OptionMap optionMap, Runnable terminationTask) throws IOException, IllegalArgumentException;
+    public XnioWorker createWorker(ThreadGroup threadGroup, OptionMap optionMap, Runnable terminationTask) throws IOException, IllegalArgumentException {
+        final XnioWorker.Builder workerBuilder = createWorkerBuilder();
+        workerBuilder.populateFromOptions(optionMap);
+        workerBuilder.setThreadGroup(threadGroup);
+        workerBuilder.setTerminationTask(terminationTask);
+        return workerBuilder.build();
+    }
 
     /**
      * Creates a file system watcher, that can be used to monitor file system changes.
