@@ -128,16 +128,23 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
         };
         final ExecutorService executorService = builder.getExternalExecutorService();
         if (executorService != null) {
-            taskPool = new ExternalTaskPool(executorService);
+            if (executorService instanceof EnhancedQueueExecutor) {
+                taskPool = new ExternalTaskPool(
+                        new EnhancedQueueExecutorTaskPool((EnhancedQueueExecutor) executorService));
+            } else if (executorService instanceof ThreadPoolExecutor) {
+                taskPool = new ExternalTaskPool(new ThreadPoolExecutorTaskPool((ThreadPoolExecutor) executorService));
+            } else {
+                taskPool = new ExternalTaskPool(new ExecutorServiceTaskPool(executorService));
+            }
         } else if (EnhancedQueueExecutor.DISABLE_HINT) {
             final int poolSize = max(builder.getMaxWorkerPoolSize(), builder.getCoreWorkerPoolSize());
-            taskPool = new ThreadPoolExecutorTaskPool(
+            taskPool = new ThreadPoolExecutorTaskPool(new DefaultThreadPoolExecutor(
                 poolSize,
                 poolSize,
                 builder.getWorkerKeepAlive(), TimeUnit.MILLISECONDS,
                 new LinkedBlockingDeque<>(),
                 new WorkerThreadFactory(builder.getThreadGroup(), builder.getWorkerStackSize(), markThreadAsDaemon),
-                terminationTask);
+                terminationTask));
         } else {
             taskPool = new EnhancedQueueExecutorTaskPool(new EnhancedQueueExecutor.Builder()
                 .setCorePoolSize(builder.getCoreWorkerPoolSize())
@@ -948,6 +955,15 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
         return taskPool.getActiveCount();
     }
 
+    /**
+     * Get an estimate of the number of threads in the worker pool.
+     *
+     * @return the estimated number of threads in the worker pool
+     */
+    protected final int getWorkerPoolSize() {
+        return taskPool.getPoolSize();
+    }
+
     /**
      * Get the maximum worker pool size.
      *
@@ -1056,8 +1072,12 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
         }
 
         public Builder addBindAddressConfiguration(CidrAddress cidrAddress, InetSocketAddress bindAddress) {
-            if (cidrAddress.getNetworkAddress().getClass() != bindAddress.getAddress().getClass()) {
-                throw Messages.msg.mismatchAddressType(cidrAddress.getNetworkAddress().getClass(), bindAddress.getAddress().getClass());
+            final Class<? extends InetAddress> networkAddrClass = cidrAddress.getNetworkAddress().getClass();
+            if (bindAddress.isUnresolved()) {
+                throw Messages.msg.addressUnresolved(bindAddress);
+            }
+            if (networkAddrClass != bindAddress.getAddress().getClass()) {
+                throw Messages.msg.mismatchAddressType(networkAddrClass, bindAddress.getAddress().getClass());
             }
             bindAddressConfigurations.put(cidrAddress, bindAddress);
             return this;
@@ -1286,13 +1306,15 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
 
         int getActiveCount();
 
+        int getPoolSize();
+
         int getQueueSize();
     }
 
-    static class ThreadPoolExecutorTaskPool extends ThreadPoolExecutor implements TaskPool {
+    static class DefaultThreadPoolExecutor extends ThreadPoolExecutor {
         private final Runnable terminationTask;
 
-        ThreadPoolExecutorTaskPool(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final TimeUnit unit, final BlockingQueue<Runnable> workQueue, final ThreadFactory threadFactory, final Runnable terminationTask) {
+        DefaultThreadPoolExecutor(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final TimeUnit unit, final BlockingQueue<Runnable> workQueue, final ThreadFactory threadFactory, final Runnable terminationTask) {
             super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory);
             this.terminationTask = terminationTask;
         }
@@ -1314,9 +1336,73 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
                 super.setMaximumPoolSize(size);
             }
         }
+    }
 
+    static class ThreadPoolExecutorTaskPool implements TaskPool {
+        private final ThreadPoolExecutor delegate;
+
+        ThreadPoolExecutorTaskPool(final ThreadPoolExecutor delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public void shutdown() {
+            delegate.shutdown();
+        }
+
+        @Override
+        public List<Runnable> shutdownNow() {
+            return delegate.shutdownNow();
+        }
+
+        @Override
+        public void execute(final Runnable command) {
+            delegate.execute(command);
+        }
+
+        @Override
+        public int getCorePoolSize() {
+            return delegate.getCorePoolSize();
+        }
+
+        @Override
+        public int getMaximumPoolSize() {
+            return delegate.getMaximumPoolSize();
+        }
+
+        @Override
+        public long getKeepAliveTime(final TimeUnit unit) {
+            return delegate.getKeepAliveTime(unit);
+        }
+
+        @Override
+        public void setCorePoolSize(final int size) {
+            delegate.setCorePoolSize(size);
+        }
+
+        @Override
+        public void setMaximumPoolSize(final int size) {
+            delegate.setMaximumPoolSize(size);
+        }
+
+        @Override
+        public void setKeepAliveTime(final long time, final TimeUnit unit) {
+            delegate.setKeepAliveTime(time, unit);
+        }
+
+        @Override
+        public int getActiveCount() {
+            return delegate.getActiveCount();
+        }
+
+        @Override
+        public int getPoolSize() {
+            return delegate.getPoolSize();
+        }
+
+        @Override
         public int getQueueSize() {
-            return getQueue().size();
+            return delegate.getQueue().size();
         }
     }
 
@@ -1367,24 +1453,28 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
             return executor.getActiveCount();
         }
 
+        public int getPoolSize() {
+            return executor.getPoolSize();
+        }
+
         public int getQueueSize() {
             return executor.getQueueSize();
         }
     }
 
-    static class ExternalTaskPool implements TaskPool {
+    static class ExecutorServiceTaskPool implements TaskPool {
         private final ExecutorService delegate;
 
-        ExternalTaskPool(final ExecutorService delegate) {
+        ExecutorServiceTaskPool(final ExecutorService delegate) {
             this.delegate = delegate;
         }
 
         public void shutdown() {
-            // no operation
+            delegate.shutdown();
         }
 
         public List<Runnable> shutdownNow() {
-            return Collections.emptyList();
+            return delegate.shutdownNow();
         }
 
         public void execute(final Runnable command) {
@@ -1416,9 +1506,81 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
             return -1;
         }
 
+        public int getPoolSize() {
+            return -1;
+        }
+
         public int getQueueSize() {
             return -1;
         }
     }
+
+    static class ExternalTaskPool implements TaskPool {
+        private final TaskPool delegate;
+
+        ExternalTaskPool(final TaskPool delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public void shutdown() {
+            // no operation
+        }
+
+        @Override
+        public List<Runnable> shutdownNow() {
+            return Collections.emptyList();
+        }
+
+        @Override
+        public void execute(final Runnable command) {
+            delegate.execute(command);
+        }
+
+        @Override
+        public int getCorePoolSize() {
+            return delegate.getCorePoolSize();
+        }
+
+        @Override
+        public int getMaximumPoolSize() {
+            return delegate.getMaximumPoolSize();
+        }
+
+        @Override
+        public long getKeepAliveTime(final TimeUnit unit) {
+            return delegate.getKeepAliveTime(unit);
+        }
+
+        @Override
+        public void setCorePoolSize(final int size) {
+            delegate.setCorePoolSize(size);
+        }
+
+        @Override
+        public void setMaximumPoolSize(final int size) {
+            delegate.setMaximumPoolSize(size);
+        }
+
+        @Override
+        public void setKeepAliveTime(final long time, final TimeUnit unit) {
+            delegate.setKeepAliveTime(time, unit);
+        }
+
+        @Override
+        public int getActiveCount() {
+            return delegate.getActiveCount();
+        }
+
+        @Override
+        public int getPoolSize() {
+            return delegate.getPoolSize();
+        }
+
+        @Override
+        public int getQueueSize() {
+            return delegate.getQueueSize();
+        }
+    }
 }
 
