@@ -43,6 +43,8 @@ import org.xnio.ByteBufferSlicePool;
 import org.xnio.ChannelListener;
 import org.xnio.Options;
 import org.xnio.Pool;
+import org.xnio.conduits.ConduitStreamSinkChannel;
+import org.xnio.conduits.ConduitStreamSourceChannel;
 import org.xnio.mock.StreamConnectionMock;
 import org.xnio.ssl.mock.SSLEngineMock.HandshakeAction;
 
@@ -53,14 +55,19 @@ import org.xnio.ssl.mock.SSLEngineMock.HandshakeAction;
  */
 public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
 
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({ "unchecked", "rawtypes" })
     @Override
     protected SslConnection createSslConnection() {
         final Pool<ByteBuffer> socketBufferPool = new ByteBufferSlicePool(BufferAllocator.BYTE_BUFFER_ALLOCATOR, 17000, 17000 * 16);
         final Pool<ByteBuffer> applicationBufferPool = new ByteBufferSlicePool(BufferAllocator.BYTE_BUFFER_ALLOCATOR, 17000, 17000 * 16);
         final StreamConnectionMock connectionMock = new StreamConnectionMock(conduitMock);
-        SslConnection connection = new JsseSslStreamConnection(connectionMock, engineMock, socketBufferPool, applicationBufferPool, true);
-        connection.getSourceChannel().getReadSetter().set(context.mock(ChannelListener.class, "read listener"));
+        final SslConnection connection = new JsseSslConnection(connectionMock, engineMock, socketBufferPool, applicationBufferPool);
+        final ChannelListener readListener = context.mock(ChannelListener.class, "read listener");
+        connection.getSourceChannel().getReadSetter().set(readListener);
+        context.checking(new Expectations() {{
+            allowing(readListener).handleEvent(with(any(ConduitStreamSinkChannel.class)));
+            allowing(readListener).handleEvent(with(any(ConduitStreamSourceChannel.class)));
+        }});
         connection.getSinkChannel().getWriteSetter().set(context.mock(ChannelListener.class, "write listener"));
         return connection;
     }
@@ -126,7 +133,7 @@ public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
         // attempt to read... channel is expected to return 0 as it stumbles upon a NEED_WRAP that cannot be executed
         assertEquals(0, sourceConduit.read(new ByteBuffer[]{buffer}, 0, 1));
         // everything is kept the same
-        assertTrue(conduitMock.isReadResumed());
+        assertFalse(conduitMock.isReadResumed());
         assertTrue(sourceConduit.isReadResumed());
         assertFalse(conduitMock.isWriteResumed());
         assertFalse(sinkConduit.isWriteResumed());
@@ -276,7 +283,7 @@ public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
         sourceConduit.resumeReads();
         assertTrue(sourceConduit.isReadResumed());
         assertFalse(conduitMock.isReadAwaken());
-        assertTrue(conduitMock.isReadResumed());
+        assertFalse(conduitMock.isReadResumed());
         sourceConduit.suspendReads();
         assertFalse(sourceConduit.isReadResumed());
         assertFalse(conduitMock.isReadResumed());
@@ -407,7 +414,6 @@ public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
         assertFalse(sinkConduit.flush());
         assertFalse(conduitMock.isWriteShutdown());
         conduitMock.enableWrites(true);
-        // FIXME workaround for bug found in SSLEngine assertFalse(sinkConduit.flush());
         assertFalse(conduitMock.isWriteShutdown());
 
         conduitMock.setReadData(CLOSE_MSG);
@@ -449,21 +455,21 @@ public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
         // start tls
         startHandshake();
 
-        // attempt to write... channel is expected to return 0 as it stumbles upon a NEED_UNWRAP that cannot be executed
-        assertEquals(0, sourceConduit.read(buffer));
-        assertFalse(conduitMock.isReadResumed());
-        assertFalse(conduitMock.isWriteResumed());
-        assertSame(HandshakeStatus.FINISHED, engineMock.getHandshakeStatus());
-        conduitMock.enableWrites(true);
+        // attempt to write... channel is expected to return 3 regardless of the fact it cannot flush
+        assertSame(HandshakeStatus.NEED_WRAP, engineMock.getHandshakeStatus());
         assertEquals(3, sourceConduit.read(new ByteBuffer[]{buffer}, 0, 1));
         assertReadMessage(buffer, "MSG");
         assertSame(HandshakeStatus.NOT_HANDSHAKING, engineMock.getHandshakeStatus());
-        assertWrittenMessage(HANDSHAKE_MSG);
+        assertWrittenMessage("");
 
+        assertFalse(sinkConduit.flush());
+        // enable writes and hence flush handshake message
+        conduitMock.enableWrites(true);
         assertTrue(sinkConduit.flush());
         assertWrittenMessage(HANDSHAKE_MSG);
 
         sourceConduit.terminateReads();
+
         assertFalse(conduitMock.isWriteShutdown());
         conduitMock.setReadData(CLOSE_MSG);
         sinkConduit.terminateWrites();
@@ -471,12 +477,6 @@ public class StartTLSConnectionTestCase extends AbstractSslConnectionTest {
         assertTrue(conduitMock.isWriteShutdown());
 
         // channel is already closed
-        assertTrue(conduitMock.isOpen());
-        assertFalse(sourceConduit.isReadShutdown());
-        assertTrue(sinkConduit.isWriteShutdown());
-        assertFalse(conduitMock.isReadShutdown());
-        assertTrue(conduitMock.isWriteShutdown());
-        connection.close();
         assertFalse(conduitMock.isOpen());
         assertTrue(sourceConduit.isReadShutdown());
         assertTrue(sinkConduit.isWriteShutdown());
