@@ -30,6 +30,7 @@ import java.lang.reflect.InvocationTargetException;
 import org.xnio.FileSystemWatcher;
 import org.xnio.IoUtils;
 import org.xnio.Options;
+import org.xnio.ReadPropertyAction;
 import org.xnio.Xnio;
 import org.xnio.OptionMap;
 import org.xnio.XnioWorker;
@@ -46,6 +47,7 @@ final class NioXnio extends Xnio {
 
     static final boolean IS_HP_UX;
     static final boolean HAS_BUGGY_EVENT_PORT;
+    static final boolean USE_ALT_QUEUED_SERVER;
 
     interface SelectorCreator {
         Selector open() throws IOException;
@@ -63,6 +65,7 @@ final class NioXnio extends Xnio {
                 return Boolean.valueOf(System.getProperty("os.name", "unknown").equalsIgnoreCase("hp-ux"));
             }
         }).booleanValue();
+        USE_ALT_QUEUED_SERVER = Boolean.parseBoolean(AccessController.doPrivileged(new ReadPropertyAction("xnio.nio.alt-queued-server", "true")));
         // if a JDK is released with a fix, we can try to detect it and set this to "false" for those JDKs.
         HAS_BUGGY_EVENT_PORT = true;
     }
@@ -75,6 +78,8 @@ final class NioXnio extends Xnio {
         final Object[] objects = AccessController.doPrivileged(
             new PrivilegedAction<Object[]>() {
                 public Object[] run() {
+                    String jdkVersion = System.getProperty("java.specification.version", "1.8");
+                    final boolean jdk9 = ! (jdkVersion.equals("1.8") || jdkVersion.equals("8"));
                     final SelectorProvider defaultProvider = SelectorProvider.provider();
                     final String chosenProvider = System.getProperty("xnio.nio.selector.provider");
                     SelectorProvider provider = null;
@@ -87,64 +92,67 @@ final class NioXnio extends Xnio {
                             provider = null;
                         }
                     }
-                    if (provider == null) {
-                        try {
-                            // Mac OS X and BSD
-                            provider = Class.forName("sun.nio.ch.KQueueSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                    if (! jdk9) {
+                        // try to probe the best available provider
+                        if (provider == null) {
+                            try {
+                                // Mac OS X and BSD
+                                provider = Class.forName("sun.nio.ch.KQueueSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
-                    }
-                    if (provider == null) {
-                        try {
-                            // Linux
-                            provider = Class.forName("sun.nio.ch.EPollSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                        if (provider == null) {
+                            try {
+                                // Linux
+                                provider = Class.forName("sun.nio.ch.EPollSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
-                    }
-                    if (provider == null && ! HAS_BUGGY_EVENT_PORT) {
-                        try {
-                            // Solaris (Java 8+)
-                            provider = Class.forName("sun.nio.ch.EventPortSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                        if (provider == null && ! HAS_BUGGY_EVENT_PORT) {
+                            try {
+                                // Solaris (Java 8+)
+                                provider = Class.forName("sun.nio.ch.EventPortSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
-                    }
-                    if (provider == null) {
-                        try {
-                            // Solaris
-                            provider = Class.forName("sun.nio.ch.DevPollSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                        if (provider == null) {
+                            try {
+                                // Solaris
+                                provider = Class.forName("sun.nio.ch.DevPollSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
-                    }
-                    if (provider == null) {
-                        try {
-                            // Solaris (Java 8+)
-                            provider = Class.forName("sun.nio.ch.EventPortSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                        if (provider == null) {
+                            try {
+                                // Solaris (Java 8+)
+                                provider = Class.forName("sun.nio.ch.EventPortSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
-                    }
-                    if (provider == null) {
-                        try {
-                            // AIX
-                            provider = Class.forName("sun.nio.ch.PollsetSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
-                            provider.openSelector().close();
-                        } catch (Throwable e) {
-                            // not available
-                            provider = null;
+                        if (provider == null) {
+                            try {
+                                // AIX
+                                provider = Class.forName("sun.nio.ch.PollsetSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
+                                provider.openSelector().close();
+                            } catch (Throwable e) {
+                                // not available
+                                provider = null;
+                            }
                         }
                     }
                     if (provider == null) {
@@ -189,7 +197,7 @@ final class NioXnio extends Xnio {
                     } catch (Exception e) {
                         // not available
                     }
-                    if (! defaultIsPoll) {
+                    if (! defaultIsPoll && ! jdk9) {
                         // default is fine for main selectors; we should try to get poll for temp though
                         if (objects[1] == null) try {
                             SelectorProvider pollSelectorProvider = Class.forName("sun.nio.ch.PollSelectorProvider", true, NioXnio.class.getClassLoader()).asSubclass(SelectorProvider.class).getConstructor().newInstance();
