@@ -30,13 +30,15 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import org.junit.Before;
+import org.junit.Test;
 import org.xnio.ChannelListener;
+import org.xnio.IoUtils;
 import org.xnio.OptionMap;
 import org.xnio.Options;
 import org.xnio.channels.ConnectedChannel;
-import org.xnio.ssl.SslConnection;
 import org.xnio.conduits.ConduitStreamSinkChannel;
 import org.xnio.conduits.ConduitStreamSourceChannel;
+import org.xnio.ssl.SslConnection;
 
 
 /**
@@ -54,81 +56,18 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
         super.setClientOptionMap(optionMap);
     }
 
-    @Override
-    public void clientClose() throws Exception {
-        log.info("Test: clientClose");
-        final CountDownLatch latch = new CountDownLatch(4);
-        final AtomicBoolean clientOK = new AtomicBoolean(false);
-        final AtomicBoolean serverOK = new AtomicBoolean(false);
-        doConnectionTest(new Runnable() {
-            public void run() {
-                try {
-                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-            }
-        }, new ChannelListener<SslConnection>() {
-            public void handleEvent(final SslConnection channel) {
-                log.info("In client open");
-                try {
-                    channel.getCloseSetter().set(new ChannelListener<SslConnection>() {
-                        public void handleEvent(final SslConnection channel) {
-                            log.info("In client close");
-                            latch.countDown();
-                        }
-                    });
-                    channel.close();
-                    clientOK.set(true);
-                    latch.countDown();
-                } catch (Throwable t) {
-                    log.error("In client", t);
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        }, new ChannelListener<SslConnection>() {
-            public void handleEvent(final SslConnection channel) {
-                log.info("In server opened");
-                channel.getCloseSetter().set(new ChannelListener<ConnectedChannel>() {
-                    public void handleEvent(final ConnectedChannel channel) {
-                        log.info("In server close");
-                        latch.countDown();
-                    }
-                });
-                setReadListener(channel, new ChannelListener<ConduitStreamSourceChannel>() {
-                    public void handleEvent(final ConduitStreamSourceChannel sourceChannel) {
-                        log.info("In server readable");
-                        try {
-                            final int c = sourceChannel.read(ByteBuffer.allocate(100));
-                            if (c == -1) {
-                                serverOK.set(true);
-                            }
-                            channel.close();
-                        } catch (IOException e) {
-                            throw new RuntimeException(e);
-                        }
-                        latch.countDown();
-                    }
-                });
-                resumeReads(channel);
-            }
-        });
-        assertTrue(serverOK.get());
-        assertTrue(clientOK.get());
-    }
-
-    @Override
-    public void oneWayTransfer1() throws Exception {
+    @Test
+    public void oneWayTransfer3() throws Exception {
         log.info("Test: oneWayTransfer");
         final CountDownLatch latch = new CountDownLatch(2);
         final AtomicInteger clientSent = new AtomicInteger(0);
         final AtomicInteger serverReceived = new AtomicInteger(0);
         final AtomicBoolean clientHandshakeStarted = new AtomicBoolean(false);
+        final AtomicBoolean serverHandshakeStarted = new AtomicBoolean(false);
         doConnectionTest(new Runnable() {
             public void run() {
                 try {
-                    assertTrue(latch.await(500000L, TimeUnit.MILLISECONDS));
+                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
                 } catch (InterruptedException e) {
                     throw new RuntimeException(e);
                 }
@@ -142,11 +81,17 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                 });
                 connection.getSinkChannel().setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
                     private boolean continueWriting() throws IOException {
-                        if (!clientHandshakeStarted.get() && clientSent.get() > 100) {
-                            if (serverReceived.get() == clientSent.get()) {
-                                connection.startHandshake();
-                                //log.info("client starting handshake");
-                                clientHandshakeStarted.set(true);
+                        if (clientSent.get() > 100) {
+                            if (!clientHandshakeStarted.get()) {
+                                if (serverReceived.get() == clientSent.get()) {
+                                    connection.startHandshake();
+                                    log.info("client starting handshake");
+                                    clientHandshakeStarted.set(true);
+                                    return true;
+                                }
+                                return false;
+                            }
+                            if (serverHandshakeStarted.get()) {
                                 return true;
                             }
                             return false;
@@ -161,6 +106,7 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                             int c;
                             try {
                                 while (continueWriting() && (c = channel.write(buffer)) > 0) {
+                                    log.info("client wrote " + (c + clientSent.get()));
                                     if (clientSent.addAndGet(c) > 1000) {
                                         final ChannelListener<ConduitStreamSinkChannel> listener = new ChannelListener<ConduitStreamSinkChannel>() {
                                             public void handleEvent(final ConduitStreamSinkChannel channel) {
@@ -171,8 +117,12 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                                                                 // really lame, but due to the way SSL shuts down...
                                                                 if (serverReceived.get() == clientSent.get()) {
                                                                     try {
+                                                                        log.info("client shutting down writes");
                                                                         channel.shutdownWrites();
-                                                                        connection.close();
+                                                                        if (connection.isWriteShutdown()) {
+                                                                            log.info("client write handler closing connection");
+                                                                            connection.close();
+                                                                        }
                                                                     } catch (Throwable t) {
                                                                         t.printStackTrace();
                                                                         throw new RuntimeException(t);
@@ -222,13 +172,19 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         try {
                             int c;
                             while ((c = channel.read(ByteBuffer.allocate(100))) > 0) {
-                                if (serverReceived.addAndGet(c) > 100) {
+                                log.info("server received " +  (c + serverReceived.get()));
+                                if (serverReceived.addAndGet(c) > 100 && !serverHandshakeStarted.get() ) {
                                     connection.startHandshake();
+                                    serverHandshakeStarted.set(true);
                                 }
                             }
                             if (c == -1) {
+                                log.info("server shutting down reads");
                                 channel.shutdownReads();
-                                connection.close();
+                                if(connection.isReadShutdown()) {
+                                    log.info("server read handler closing connection");
+                                    connection.close();
+                                }
                             }
                         } catch (Throwable t) {
                             t.printStackTrace();
@@ -242,17 +198,17 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
         assertEquals(clientSent.get(), serverReceived.get());
     }
 
-    @Override
-    public void oneWayTransfer2() throws Exception {
-        log.info("Test: oneWayTransfer2");
+    public void oneWayTransfer4() throws Exception {
+        log.info("Test: oneWayTransfer4");
         final CountDownLatch latch = new CountDownLatch(2);
         final AtomicInteger clientReceived = new AtomicInteger(0);
         final AtomicInteger serverSent = new AtomicInteger(0);
+        final AtomicBoolean clientHandshakeStarted = new AtomicBoolean(false);
         final AtomicBoolean serverHandshakeStarted = new AtomicBoolean(false);
         doConnectionTest(new Runnable() {
             public void run() {
                 try {
-                    assertTrue(latch.await(500000L, TimeUnit.MILLISECONDS));
+                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
                 } catch (InterruptedException e) {
                     throw new RuntimeException(e);
                 }
@@ -269,13 +225,16 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         try {
                             int c;
                             while ((c = channel.read(ByteBuffer.allocate(100))) > 0) {
-                                if (clientReceived.addAndGet(c) > 100) {
+                                log.info("client received " +  (c + clientReceived.get()));
+                                if (clientReceived.addAndGet(c) > 100 && !clientHandshakeStarted.get()) {
                                     connection.startHandshake();
+                                    clientHandshakeStarted.set(true);
                                 }
                             }
                             if (c == -1) {
                                 channel.shutdownReads();
-                                connection.close();
+                                if (connection.isReadShutdown())
+                                    connection.close();
                             }
                         } catch (Throwable t) {
                             t.printStackTrace();
@@ -295,11 +254,17 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                 });
                 connection.getSinkChannel().setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
                     private boolean continueWriting() throws IOException {
-                        if (!serverHandshakeStarted.get() && serverSent.get() > 100) {
-                            if (clientReceived.get() == serverSent.get()) {
-                                connection.startHandshake();
-                                //log.info("client starting handshake");
-                                serverHandshakeStarted.set(true);
+                        if (serverSent.get() > 100) {
+                            if (!serverHandshakeStarted.get()) {
+                                if (clientReceived.get() == serverSent.get()) {
+                                    connection.startHandshake();
+                                    log.info("server starting handshake");
+                                    serverHandshakeStarted.set(true);
+                                    return true;
+                                }
+                                return false;
+                            }
+                            if (clientHandshakeStarted.get()) {
                                 return true;
                             }
                             return false;
@@ -314,6 +279,7 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                             int c;
                             try {
                                 while (continueWriting() && (c = channel.write(buffer)) > 0) {
+                                    log.info("server wrote " + (c + serverSent.get()));
                                     if (serverSent.addAndGet(c) > 100) {
                                         connection.startHandshake();
                                         if (serverSent.get() > 1000) {
@@ -327,7 +293,8 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                                                                     if (clientReceived.get() == serverSent.get()) {
                                                                         try {
                                                                             channel.shutdownWrites();
-                                                                            connection.close();
+                                                                            if (connection.isWriteShutdown())
+                                                                                connection.close();
                                                                         } catch (Throwable t) {
                                                                             t.printStackTrace();
                                                                             throw new RuntimeException(t);
@@ -368,9 +335,9 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
         assertEquals(serverSent.get(), clientReceived.get());
     }
 
-    @Override
-    public void twoWayTransfer() throws Exception {
-        log.info("Test: twoWayTransfer");
+    @Test
+    public void twoWayTransferWithHandshake() throws Exception {
+        log.info("Test: twoWayTransferWithHandshake");
         final CountDownLatch latch = new CountDownLatch(2);
         final AtomicInteger clientSent = new AtomicInteger(0);
         final AtomicInteger clientReceived = new AtomicInteger(0);
@@ -381,7 +348,7 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
         doConnectionTest(new Runnable() {
             public void run() {
                 try {
-                    assertTrue(latch.await(500000L, TimeUnit.MILLISECONDS));
+                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
                 } catch (InterruptedException e) {
                     throw new RuntimeException(e);
                 }
@@ -389,42 +356,48 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
         }, new ChannelListener<SslConnection>() {
             public void handleEvent(final SslConnection connection) {
                 connection.getCloseSetter().set(new ChannelListener<SslConnection>() {
-                    public void handleEvent(final SslConnection channel) {
+                    public void handleEvent(final SslConnection connection) {
                         latch.countDown();
                     }
                 });
-                connection.getSourceChannel().setReadListener(new ChannelListener<ConduitStreamSourceChannel>() {
+                final ConduitStreamSourceChannel sourceChannel = connection.getSourceChannel();
+                sourceChannel.setReadListener(new ChannelListener<ConduitStreamSourceChannel>() {
 
                     private boolean continueReading() throws IOException {
                         return clientHandshakeStarted.get() || clientReceived.get() < 101;
                     }
 
-                    public void handleEvent(final ConduitStreamSourceChannel channel) {
-                        //log.info("client handle read events");
+                    public void handleEvent(final ConduitStreamSourceChannel sourceChannel) {
+                        log.info("client handle read events");
                         try {
                             int c = 0;
-                            while (continueReading() && (c = channel.read(ByteBuffer.allocate(100))) > 0) {
-                                //log.info("client received: "+ (clientReceived.get() + c));
+                            while (continueReading() && (c = sourceChannel.read(ByteBuffer.allocate(100))) > 0) {
+                                log.info("client received: "+ (clientReceived.get() + c));
                                 clientReceived.addAndGet(c);
                             }
                             if (c == -1) {
-                                //log.info("client shutdown reads");
-                                channel.shutdownReads();
+                                log.info("client shutdown reads");
+                                connection.close();
                             }
                         } catch (Throwable t) {
                             t.printStackTrace();
-                            throw new RuntimeException(t);
                         }
                     }
                 });
-                connection.getSinkChannel().setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
-
-                    private boolean continueWriting(ConduitStreamSinkChannel channel) throws IOException {
-                        if (!clientHandshakeStarted.get() && clientSent.get() > 100) {
-                            if (serverReceived.get() == clientSent.get() && serverSent.get() > 100 && clientReceived.get() == serverSent.get() ) {
-                                connection.startHandshake();
-                                //log.info("client starting handshake");
-                                clientHandshakeStarted.set(true);
+                final ConduitStreamSinkChannel sinkChannel = connection.getSinkChannel();
+                sinkChannel.setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
+                    private boolean continueWriting(ConduitStreamSinkChannel sinkChannel) throws IOException {
+                        if (clientSent.get() > 100) {
+                            if (!clientHandshakeStarted.get()) {
+                                if (serverReceived.get() == clientSent.get() && serverSent.get() > 100 && clientReceived.get() == serverSent.get() ) {
+                                    connection.startHandshake();
+                                    log.info("client starting handshake");
+                                    clientHandshakeStarted.set(true);
+                                    return true;
+                                }
+                                return false;
+                            }
+                            if (clientHandshakeStarted.get()) {
                                 return true;
                             }
                             return false;
@@ -432,35 +405,26 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         return true;
                     }
 
-                    public void handleEvent(final ConduitStreamSinkChannel channel) {
+                    public void handleEvent(final ConduitStreamSinkChannel sinkChannel) {
                                                 try {
                             final ByteBuffer buffer = ByteBuffer.allocate(100);
                             buffer.put("This Is A Test\r\n".getBytes("UTF-8")).flip();
                             int c = 0;
                             try {
-                                while (continueWriting(channel) && (clientSent.get() > 1000 || (c = channel.write(buffer)) > 0)) {
-                                    //log.info("clientSent: " + (clientSent.get() + c));
+                                while (continueWriting(sinkChannel) && (clientSent.get() > 1000 || (c = sinkChannel.write(buffer)) > 0)) {
+                                    log.info("clientSent: " + (clientSent.get() + c));
                                     if (clientSent.addAndGet(c) > 1000) {
-                                        final ChannelListener<ConduitStreamSinkChannel> listener = new ChannelListener<ConduitStreamSinkChannel>() {
-                                            public void handleEvent(final ConduitStreamSinkChannel channel) {
+                                        sinkChannel.setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
+                                            public void handleEvent(final ConduitStreamSinkChannel sinkChannel) {
                                                 try {
-                                                    if (channel.flush()) {
-                                                        final ChannelListener<ConduitStreamSinkChannel> listener = new ChannelListener<ConduitStreamSinkChannel>() {
-                                                            public void handleEvent(final ConduitStreamSinkChannel channel) {
-                                                                // really lame, but due to the way SSL shuts down...
-                                                                if (clientReceived.get() == serverSent.get() && serverReceived.get() == clientSent.get() && serverSent.get() > 1000) {
-                                                                    try {
-                                                                        //log.info("client closing channel");
-                                                                        connection.close();
-                                                                    } catch (Throwable t) {
-                                                                        t.printStackTrace();
-                                                                        throw new RuntimeException(t);
-                                                                    }
-                                                                }
-                                                            }
-                                                        };
-                                                        channel.setWriteListener(listener);
-                                                        listener.handleEvent(channel);
+                                                    if (sinkChannel.flush()) {
+                                                        try {
+                                                            log.info("client closing channel");
+                                                            sinkChannel.shutdownWrites();
+                                                        } catch (Throwable t) {
+                                                            t.printStackTrace();
+                                                            throw new RuntimeException(t);
+                                                        }
                                                         return;
                                                     }
                                                 } catch (Throwable t) {
@@ -468,16 +432,14 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                                                     throw new RuntimeException(t);
                                                 }
                                             }
-                                        };
-                                        channel.getWriteSetter().set(listener);
-                                        listener.handleEvent(channel);
+                                        });
                                         return;
                                     }
+                                    buffer.rewind();
                                 }
-                                buffer.rewind();
                             } catch (ClosedChannelException e) {
                                 try {
-                                    channel.shutdownWrites();
+                                    sinkChannel.shutdownWrites();
                                 } catch (Exception cce) {/* do nothing */}
                                 throw e;
                             }
@@ -488,8 +450,8 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                     }
                 });
 
-                connection.getSourceChannel().resumeReads();
-                connection.getSinkChannel().resumeWrites();
+                sourceChannel.resumeReads();
+                sinkChannel.resumeWrites();
             }
         }, new ChannelListener<SslConnection>() {
             public void handleEvent(final SslConnection connection) {
@@ -498,21 +460,22 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         latch.countDown();
                     }
                 });
-                connection.getSourceChannel().setReadListener(new ChannelListener<ConduitStreamSourceChannel>() {
+                final ConduitStreamSourceChannel sourceChannel = connection.getSourceChannel();
+                sourceChannel.setReadListener(new ChannelListener<ConduitStreamSourceChannel>() {
                     private boolean continueReading() throws IOException {
                         return serverHandshakeStarted.get() || serverReceived.get() < 101;
                     }
 
-                    public void handleEvent(final ConduitStreamSourceChannel channel) {
+                    public void handleEvent(final ConduitStreamSourceChannel sourceChannel) {
                         try {
                             int c = 0;
-                            while (continueReading() && (c = channel.read(ByteBuffer.allocate(100))) > 0) {
-                                //log.info("server received: "+ (serverReceived.get() + c));
+                            while (continueReading() && (c = sourceChannel.read(ByteBuffer.allocate(100))) > 0) {
+                                log.info("server received: "+ (serverReceived.get() + c));
                                 serverReceived.addAndGet(c);
                             }
                             if (c == -1) {
-                                //log.info("server shutdown reads");
-                                channel.shutdownReads();
+                                log.info("server shutdown reads");
+                                connection.close();
                             }
                         } catch (Throwable t) {
                             t.printStackTrace();
@@ -520,14 +483,21 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         }
                     }
                 });
-                connection.getSinkChannel().setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
+                final ConduitStreamSinkChannel sinkChannel = connection.getSinkChannel();
+                sinkChannel.setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
 
-                    private boolean continueWriting() throws IOException {
-                        if (!serverHandshakeStarted.get() && serverSent.get() > 100) {
-                            if (clientReceived.get() == serverSent.get() && clientSent.get() > 100 && serverReceived.get() == clientSent.get() ) {
-                                //log.info("server starting handshake");
-                                connection.startHandshake();
-                                serverHandshakeStarted.set(true);
+                    private boolean continueWriting(ConduitStreamSinkChannel sinkChannel) throws IOException {
+                        if (serverSent.get() > 100) {
+                            if (!serverHandshakeStarted.get()) {
+                                if (clientReceived.get() == serverSent.get() && clientSent.get() > 100 && serverReceived.get() == clientSent.get() ) {
+                                    connection.startHandshake();
+                                    log.info("server starting handshake");
+                                    serverHandshakeStarted.set(true);
+                                    return true;
+                                }
+                                return false;
+                            }
+                            if (clientHandshakeStarted.get()) {
                                 return true;
                             }
                             return false;
@@ -535,35 +505,26 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         return true;
                     }
 
-                    public void handleEvent(final ConduitStreamSinkChannel channel) {
+                    public void handleEvent(final ConduitStreamSinkChannel sinkChannel) {
                         try {
                             final ByteBuffer buffer = ByteBuffer.allocate(100);
                             buffer.put("This Is A Test\r\n".getBytes("UTF-8")).flip();
                             int c;
                             try {
-                                while (continueWriting() && (c = channel.write(buffer)) > 0) {
-                                    //log.info("server sent: "+ (serverSent.get() + c));
+                                while (continueWriting(sinkChannel) && (c = sinkChannel.write(buffer)) > 0) {
+                                    log.info("server sent: "+ (serverSent.get() + c));
                                     if (serverSent.addAndGet(c) > 1000) {
-                                        final ChannelListener<ConduitStreamSinkChannel> listener = new ChannelListener<ConduitStreamSinkChannel>() {
-                                            public void handleEvent(final ConduitStreamSinkChannel channel) {
+                                        sinkChannel.setWriteListener(new ChannelListener<ConduitStreamSinkChannel>() {
+                                            public void handleEvent(final ConduitStreamSinkChannel sinkChannel) {
                                                 try {
-                                                    if (channel.flush()) {
-                                                        final ChannelListener<ConduitStreamSinkChannel> listener = new ChannelListener<ConduitStreamSinkChannel>() {
-                                                            public void handleEvent(final ConduitStreamSinkChannel channel) {
-                                                                // really lame, but due to the way SSL shuts down...
-                                                                if (clientReceived.get() == serverSent.get() && serverReceived.get() == clientSent.get() && clientSent.get() > 1000) {
-                                                                    try {
-                                                                        //log.info("server closing channel");
-                                                                        connection.close();
-                                                                    } catch (Throwable t) {
-                                                                        t.printStackTrace();
-                                                                        throw new RuntimeException(t);
-                                                                    }
-                                                                }
-                                                            }
-                                                        };
-                                                        channel.getWriteSetter().set(listener);
-                                                        listener.handleEvent(channel);
+                                                    if (sinkChannel.flush()) {
+                                                        try {
+                                                            log.info("server closing channel");
+                                                            sinkChannel.shutdownWrites();
+                                                        } catch (Throwable t) {
+                                                            t.printStackTrace();
+                                                            throw new RuntimeException(t);
+                                                        }
                                                         return;
                                                     }
                                                 } catch (Throwable t) {
@@ -571,15 +532,13 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                                                     throw new RuntimeException(t);
                                                 }
                                             }
-                                        };
-                                        channel.setWriteListener(listener);
-                                        listener.handleEvent(channel);
+                                        });
                                         return;
                                     }
                                 }
                                 buffer.rewind();
                             } catch (ClosedChannelException e) {
-                                channel.shutdownWrites();
+                                sinkChannel.shutdownWrites();
                                 throw e;
                             }
                         } catch (Throwable t) {
@@ -588,8 +547,8 @@ public class NioStartTLSTcpConnectionTestCase extends NioSslTcpConnectionTestCas
                         }
                     }
                 });
-                connection.getSourceChannel().resumeReads();
-                connection.getSinkChannel().resumeWrites();
+                sourceChannel.resumeReads();
+                sinkChannel.resumeWrites();
             }
         });
         assertEquals(serverSent.get(), clientReceived.get());
